// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * @module     mod_capquiz
 * @author     Sebastian S. Gundersen <sebastian@sgundersen.com>
 * @copyright  2019 NTNU
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

define(['jquery'], function($) {

    var parameters = {
        courseModuleId: 0,
    };

    /**
     * Send an action to the server.
     * @param {Object} data
     * @param {Object} onSuccess
     * @param {Object} onError
     */
    function sendAction(data, onSuccess, onError) {
        $.ajax({
            type: 'post',
            url: 'action.php',
            data: data,
            success: onSuccess,
            error: onError
        });
    }

    /**
     * Send the new default rating for the question list to the server.
     * @param {Object} data
     * @param {number} rating
     * @param {Object} onSuccess
     * @param {Object} onError
     */
    function sendDefaultQuestionRating(data, rating, onSuccess, onError) {
        sendAction({
            'action': 'set-default-question-rating',
            'id': parameters.courseModuleId,
            'rating': rating,
        }, onSuccess, onError);
    }

    /**
     * Send the new rating for the question to the server.
     * @param {Object} data
     * @param {number} rating
     * @param {Object} onSuccess
     * @param {Object} onError
     */
    function sendQuestionRating(data, rating, onSuccess, onError) {
        sendAction({
            'action': 'set-question-rating',
            'id': parameters.courseModuleId,
            'question-id': data.questionId,
            'rating': rating,
        }, onSuccess, onError);
    }

    /**
     * Send the new value, and avoid race condition.
     * @param {Object} $input
     * @param {Object} sendInput
     * @param {Object} data
     */
    function submitInput($input, sendInput, data) {
        $input.data('saving', true);
        $input.data('dirty', false);
        var $indicator = $input.next();
        $indicator.css('color', 'blue');
        sendInput(data, $input.val(), function() {
            if ($input.data('dirty') === true) {
                submitInput($input, sendInput, data);
            } else {
                $indicator.css('color', 'green');
                $input.data('dirty', false);
                $input.data('saving', false);
            }
        }, function() {
            $indicator.css('color', 'red');
        });
    }

    /**
     * Send the new rating for the question, and avoid race condition.
     * @param {Object} $input
     */
    function submitQuestionRating($input) {
        submitInput($input, sendQuestionRating, {questionId: $input.data('question-id')});
    }

    /**
     * Send the new default rating for the question list, and avoid race condition.
     * @param {Object} $input
     */
    function submitDefaultQuestionRating($input) {
        submitInput($input, sendDefaultQuestionRating, null);
    }

    /**
     * Register an input event listener for submission.
     * @param {string} query
     * @param {Object} submit
     */
    function registerListener(query, submit) {
        $(document).on('input', query, function(event) {
            var $input = $(event.target);
            var isBeingSaved = $input.data('saving');
            if (isBeingSaved === true) {
                $input.data('dirty', true);
                return;
            }
            submit($input);
        });
    }

    /**
     * Sorts a table by the respective column based on $header.
     * It searches for an element of class "capquiz-sortable-item" inside the <td>, and if found,
     * the value attribute is used if it exists. Otherwise, the inner html is used to sort by.
     *
     * The <td> tag may not have the item class, as it has no effect on the sorting.
     * Their children elements are not required to have the class either. The inner html of <td> will be used then.
     *
     * The first column in the table must be an index of the row.
     *
     * @param {Object} $header The header column for which to sort the table by.
     */
    function sortTable($header) {
        const column = $header.index();
        const $table = $header.parent().parent();
        let $rows = $table.find('tr:gt(0)').toArray().sort(function(rowA, rowB) {
            const $colA = $(rowA).children('td').eq(0);
            const $colB = $(rowB).children('td').eq(0);
            return parseInt($colA.text()) - parseInt($colB.text());
        });
        $table.append($rows);
        $rows = $table.find('tr:gt(0)').toArray().sort(function(rowA, rowB) {
            const $colA = $(rowA).children('td').eq(column);
            const $colB = $(rowB).children('td').eq(column);
            const $itemA = $colA.find('.capquiz-sortable-item');
            const $itemB = $colB.find('.capquiz-sortable-item');
            let valA;
            if ($itemA.length === 0) {
                valA = $colA.html();
            } else {
                valA = $itemA.val().length === 0 ? $itemA.html() : $itemA.val();
            }
            let valB;
            if ($itemB.length === 0) {
                valB = $colB.html();
            } else {
                valB = $itemB.val().length === 0 ? $itemB.html() : $itemB.val();
            }
            if ($.isNumeric(valA) && $.isNumeric(valB)) {
                return valA - valB;
            } else {
                return valA.toString().localeCompare(valB);
            }
        });
        const ascending = ($table.data('asc') === 'true');
        $table.data('asc', ascending ? 'false' : 'true');
        const iconName = (ascending ? 'fa-arrow-up' : 'fa-arrow-down');
        $.each($table.find('.capquiz-sortable'), function() {
            $(this).find('.fa').remove();
        });
        $header.prepend('<i class="fa ' + iconName + '"></i>');
        if (!ascending) {
            $rows = $rows.reverse();
        }
        $table.append($rows);
        let i = 1;
        $table.find('tr:gt(0)').each(function() {
            $(this).find('td:first-child').html(i);
            i++;
        });
    }

    /**
     * Register click event listeners for the sortable table columns.
     */
    function registerSortListener() {
        $(document).on('click', '.capquiz-sortable', function() {
            sortTable($(this));
        });
        $('.capquiz-sortable-default-asc').each(function() {
            sortTable($(this));
            sortTable($(this));
        });
        $('.capquiz-sortable-default-desc').each(function() {
            sortTable($(this));
        });
    }

    /**
     * Set the tab indices for the question rating elements to be more user friendly.
     */
    function fixTabIndicesForQuestionRatingInputs() {
        $('.capquiz-question-rating-submit-wrapper button').each(function(index, object) {
            $(object).attr('tabindex', -1);
        });
    }

    /**
     * Register click event listener for "Add to quiz" button.
     */
    function listenAddToQuiz() {
        $('.capquiz-add-selected-questions').on('click', function() {
            var questionIds = '';
            $('#categoryquestions td input[type=checkbox]:checked').each(function() {
                questionIds += $(this).attr('name').slice(1) + ',';
            });
            $.post('action.php', {
                'action': 'add-question',
                'id': parameters.courseModuleId,
                'question-id': questionIds,
            }, function() {
                location.reload();
            });
        });
    }

    return {
        initialize: function(courseModuleId) {
            parameters.courseModuleId = courseModuleId;
            registerListener('.capquiz-question-rating input', submitQuestionRating);
            registerListener('.capquiz-default-question-rating input', submitDefaultQuestionRating);
            fixTabIndicesForQuestionRatingInputs();
            registerSortListener();
            listenAddToQuiz();
        }
    };

});
